(*****************************************************************************)
(* Lexer for Lysa ASCII parser                                               *)
(*                                 Time-stamp: <14-01-2004 Mikael Buchholtz> *)
(*****************************************************************************)


(* Declarations for lexer structures *)
type pos            = int
type svalue         = Tokens.svalue
type ('a, 'b) token = ('a, 'b) Tokens.token 


(* Position information *)
val pos             = ref 1

(* ML-lex required definitions *)
type lexresult      = (Tokens.svalue,int) Tokens.token
fun eof ()          = (pos:=1;Tokens.EOF(!pos,!pos))

(* Error handling *)
exception lexError of string
fun error s         =  raise lexError ("line "^Int.toString(!pos)^": "^s)


%%

%header (functor LysaLexerFun(structure Tokens : Lysa_TOKENS) );

%s COMMENT;

alpha    = [A-Za-z];
num      = [0-9];
ws       = [\ \t];
alphanum = {alpha}|{num};

%%
\n                            => (pos := (!pos)+1; lex());
{ws}+                         => (lex());
<INITIAL> "/*"                => (YYBEGIN COMMENT; lex());
<COMMENT> "*/"                => (YYBEGIN INITIAL; lex());
<COMMENT> .                   => (lex());
<INITIAL> "("                 => (Tokens.LBRACKET(!pos,!pos));
<INITIAL> ")"                 => (Tokens.RBRACKET(!pos,!pos));
<INITIAL> "{"                 => (Tokens.LCBRACKET(!pos,!pos));
<INITIAL> "}"                 => (Tokens.RCBRACKET(!pos,!pos));
<INITIAL> "["                 => (Tokens.LSBRACKET(!pos,!pos));
<INITIAL> "]"                 => (Tokens.RSBRACKET(!pos,!pos));
<INITIAL> "<"                 => (Tokens.LABRACKET(!pos,!pos));
<INITIAL> ">"                 => (Tokens.RABRACKET(!pos,!pos));
<INITIAL> "|_"                => (Tokens.IBAR(!pos,!pos));
<INITIAL> "="                 => (Tokens.EQUAL(!pos,!pos));
<INITIAL> "."                 => (Tokens.POINT(!pos,!pos));
<INITIAL> ","                 => (Tokens.COMMA(!pos,!pos));
<INITIAL> ";"                 => (Tokens.SEMICOLON(!pos,!pos));
<INITIAL> ":"                 => (Tokens.COLON(!pos,!pos));
<INITIAL> "_"                 => (Tokens.UNDERSCORE(!pos,!pos));
<INITIAL> "|"                 => (Tokens.BAR(!pos,!pos));
<INITIAL> "!"                 => (Tokens.BANG(!pos,!pos));
<INITIAL> "+"                 => (Tokens.PLUS(!pos,!pos));
<INITIAL> "-"                 => (Tokens.MINUS(!pos,!pos));
<INITIAL> \\                  => (Tokens.BACKSLASH(!pos,!pos));
<INITIAL> new                 => (Tokens.NEW(!pos,!pos));
<INITIAL> decrypt             => (Tokens.DECRYPT(!pos,!pos));
<INITIAL> as                  => (Tokens.AS(!pos,!pos));
<INITIAL> in                  => (Tokens.IN(!pos,!pos));
<INITIAL> 0                   => (Tokens.NIL(!pos,!pos));
<INITIAL> orig                => (Tokens.ORIG(!pos,!pos));
<INITIAL> dest                => (Tokens.DEST(!pos,!pos));
<INITIAL> at                  => (Tokens.AT(!pos,!pos));
<INITIAL> CPDY                => (Tokens.CPDY(!pos,!pos));
<INITIAL> -?{num}+            => (case Int.fromString(yytext) of 
				     SOME(i) => Tokens.num(i,!pos,!pos)
				  |  NONE    => (error("Unanticipated problem with number convesion.\n");
						 Tokens.num(0,!pos,!pos)));
<INITIAL> {alpha}{alphanum}*  => (Tokens.tid(yytext,!pos,!pos));
<INITIAL> .                   => (error("Unexpected character: "^yytext);lex());
